/*
 * Copyright (c) 2000, 2001 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: bktr.c,v 1.28 2001/07/15 12:42:09 pva Exp $
 *  Driver for cards support for which is compiled into kernel
 *  BSD part of the driver is a very early ALPHA!!!
 */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <err.h>
#include <sys/stat.h>
#include <sys/ioctl.h>

#if defined (__FreeBSD__)
#include <machine/ioctl_bt848.h>
#elif defined (__NetBSD__) 
#include <dev/ic/bt8xx.h>
#elif defined (__OpenBSD__)
#include <sys/param.h>
#if OpenBSD < 200105
#include <machine/ioctl_bt848.h>
#else
#include <dev/ic/bt8xx.h>
#endif /* OpenBSD < 200105 */
#elif defined linux
#include <sys/io.h>
#include <linux/videodev.h>
#endif

#include "io.h"

static int stereo = 1; /* Use stereo by default */
static int fd = -1;
static int vol = 0;

#if defined (__FreeBSD__) || defined (__OpenBSD__) || defined (__NetBSD__)
#define TUNER_DEVICE_1	"/dev/tuner"
#define TUNER_DEVICE_2	"/dev/tuner0"
#elif defined linux
#define TUNER_DEVICE_1	"/dev/radio"
#define TUNER_DEVICE_2	"/dev/radio0"
static double get_freq_fact(int);
static int tuner_ord = 0;
#endif

#define SYMLINK_DEPTH	10
#define MAX_FILENAME	1024

/******************************************************************/

int
Grab_Port_bktr(unsigned long port) {
#if defined (__FreeBSD__) || defined (__OpenBSD__) || defined (__NetBSD__)
	int intern = AUDIO_INTERN;
#elif defined linux
	struct video_tuner t;
#endif /* __FreeBSD__ || __OpenBSD__ || __NetBSD__ */
	char buf[MAX_FILENAME], next_buf[MAX_FILENAME];
	struct stat s;
	int rfnb = 0; /* Length of file name read from readlink() */
	int depth = 0, found = 0;

	strcpy(buf, TUNER_DEVICE_1);

	/* Follow symlinks until the file is found */
	for ( depth = 0; depth < SYMLINK_DEPTH; depth++ ) {

		if ( lstat(buf, &s) < 0 )
			break;

		if ( !S_ISLNK(s.st_mode) ) {
			found = 1;
			break;
		}

		rfnb = readlink(buf, next_buf, MAX_FILENAME-1);
		if ( rfnb < 0 || rfnb == 0 )
			break;
		next_buf[rfnb] = '\0';

		strlcpy(buf, next_buf, MAX_FILENAME);
		buf[MAX_FILENAME-1] = '\0';
	}

	/*
	 * Ok, TUNER_DEVICE_1 doesn't exist or is buried too deep
	 * Let's try last resort -- TUNER_DEVICE_2
	 */
	if ( !found )
		strcpy(buf, TUNER_DEVICE_2);

	if ( (fd = open(buf, O_RDONLY)) < 0 )
		return 1;

	/* Check for working driver */
#if defined (__FreeBSD__) || defined (__OpenBSD__) || defined (__NetBSD__)
	if ( ioctl(fd, BT848_SAUDIO, &intern) < 0 )
#elif defined linux
	/* FIXME: it can be not the first tuner */
	t.tuner = tuner_ord;

	if ( ioctl(fd, VIDIOCGTUNER, &t) < 0 )
#endif
		return 1;
	return 0;
}

int
Release_Port_bktr(void) {
	if ( close(fd) < 0 )
		return 1;

	return 0;
}

void
Change_Frequency_bktr(unsigned long frequency) {
#if defined (__FreeBSD__) || defined (__OpenBSD__) || defined (__NetBSD__)
	if ( ioctl(fd, RADIO_SETFREQ, &frequency) < 0 )
#elif defined linux
	unsigned long freq = (unsigned long)(frequency*get_freq_fact(fd));

	if ( ioctl(fd, VIDIOCSFREQ, &freq) < 0 )
#endif
		warn("frequency set error");
}

void
mute_bktr(void) {
#if defined (__FreeBSD__) || defined (__OpenBSD__) || defined (__NetBSD__)
	int intern = AUDIO_MUTE;

	if ( !vol ) return;

	vol = 0;

	if ( ioctl(fd, BT848_SAUDIO, &intern) < 0 )
#elif defined linux
	struct video_audio va;

	va.audio = 0;
	va.flags = VIDEO_AUDIO_MUTE;
	va.volume = vol = 0;

	if ( ioctl(fd, VIDIOCSAUDIO, &va) < 0 )
#endif
		warn("mute error");
}

void
volumeup_bktr(void) {
#if defined (__FreeBSD__) || defined (__OpenBSD__) || defined (__NetBSD__)
	int intern = AUDIO_UNMUTE;

	if ( vol ) return;

	vol = 1;

	if ( ioctl(fd, BT848_SAUDIO, &intern) < 0 )
		warn("unmute error");
#elif defined linux
	if ( vol < 10 ) {
		struct video_audio va;
		
		va.flags = VIDEO_AUDIO_VOLUME;
		if ( stereo )
			va.mode = VIDEO_SOUND_STEREO;
		else
			va.mode = VIDEO_SOUND_MONO;
		va.audio = 0;
		va.volume = (++vol)*(65535/10);

		if ( ioctl(fd, VIDIOCSAUDIO, &va) < 0 )
			warn("volume up error");
	}
#endif
}

void
volumedn_bktr(void) {
#if defined (__FreeBSD__) || defined (__OpenBSD__) || defined (__NetBSD__)
	int intern = AUDIO_MUTE;

	if ( !vol ) return;

	vol = 0;

	if ( ioctl(fd, BT848_SAUDIO, &intern) < 0)
		warn("mute error");
#elif defined linux
	if ( vol > 0 ) {
		struct video_audio va;

		va.flags = VIDEO_AUDIO_VOLUME;
		if ( stereo )
			va.mode = VIDEO_SOUND_STEREO;
		else
			va.mode = VIDEO_SOUND_MONO;
		va.audio = 0;
		va.volume = (--vol)*(65535/10);

		if ( ioctl(fd, VIDIOCSAUDIO, &va) < 0 )
			warn("volume down error");
	}
#endif
}

int
state_bktr(void) {
#if defined (__FreeBSD__) || defined (__OpenBSD__) || defined (__NetBSD__)
	int ss;

	if ( ioctl(fd, TVTUNER_GETSTATUS, &ss) < 0 ) return 3;

	ss &= 0x07;

	if ( ss > 0x06 ) return 0;
	if ( ss > 0x03 ) return 1;
	if ( ss > 0x01 ) return 2;
#elif defined linux
	struct video_tuner t;

	t.tuner = tuner_ord;

	if ( ioctl(fd, VIDIOCGTUNER, &t) < 0 ) return 3;

	if ( t.flags & VIDEO_TUNER_STEREO_ON ) return 0; /* STEREO */
	if ( t.signal > 49061 ) return 1; /* almost stereo */
	if ( t.signal > 32678 ) return 2; /* mono */
#endif
	return 3;
}

void
mono_bktr(void) {
#if defined (__FreeBSD__) || defined (__OpenBSD__) || defined (__NetBSD__)
	unsigned int mode = RADIO_MONO;

	if ( !stereo ) return;

	stereo = 0;

	if ( ioctl(fd, RADIO_SETMODE, &mode) < 0)
#elif defined linux
	struct video_audio va;

	va.audio = 0;
	va.mode = VIDEO_SOUND_MONO;
	stereo = 0;

	if ( ioctl(fd, VIDIOCSAUDIO, &va) < 0 )
#endif
		warn("mono set error");
}

#ifdef linux
static double
get_freq_fact(int fd) {
	struct video_tuner t;
	t.tuner = tuner_ord;
	if ( ioctl(fd, VIDIOCGTUNER, &t) == -1 || (t.flags & VIDEO_TUNER_LOW) == 0 )
		return .16;

	return 160.;
}
#endif /* linux */
