/*
 * Copyright (c) 2001 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: gemtek.c,v 1.21 2001/05/15 09:54:53 pva Exp $
 *
 * lowlevel io functions for fmio utility -- gemtek driver
 */

#include <unistd.h>

#ifdef linux
#include <sys/io.h>
#elif defined __FreeBSD__
#include <fcntl.h>
#include <machine/cpufunc.h>
#else
#include <machine/pio.h>
#include <machine/sysarch.h>
#endif

#include "io.h"

static void send_zero(void);
static void send_one(void);
static void send_init(void);

static unsigned long radioport = 0;

#ifdef __FreeBSD__
static int iofd;
#endif /* __FreeBSD__ */

/*********************************************************************/

int
Grab_Port_gemtek(unsigned long port) {
#if defined (__OpenBSD__) || defined (__NetBSD__)
	unsigned long iomap[32];
	int offset;
	unsigned long mask;
	struct i386_set_ioperm_args ioperms;
#endif /* __OpenBSD__ || __NetBSD__ */

	radioport = port;

#ifdef linux /* Platform dependent */
	if ( ioperm(radioport, 1, 1) < 0 )
#elif defined __FreeBSD__
	if ( (iofd = open("/dev/io", O_RDONLY)) < 0 )
#else
	offset = radioport/32;
	mask = 1 << radioport%32;
	memset(iomap, 0xFFFF, sizeof(iomap));
	iomap[offset] ^= mask;
	ioperms.iomap = iomap;
	if ( sysarch(I386_SET_IOPERM, (char *)&ioperms) < 0 )
#endif /* Platform dependent */
		return 1;

	return 0;
}

int
Release_Port_gemtek(void) {
#if defined (__OpenBSD__) || defined (__NetBSD__)
	unsigned long iomap[32];
	struct i386_set_ioperm_args ioperms;
#endif /* __OpenBSD__ || __NetBSD__ */

#ifdef linux /* Platform dependent */
	if ( ioperm(radioport, 1, 0) < 0 )
#elif defined __FreeBSD__
	if ( close(iofd) < 0 )
#else
	memset(iomap, 0xFFFF, sizeof(iomap));
	ioperms.iomap = iomap;
	if ( sysarch(I386_SET_IOPERM, (char *)&ioperms) < 0 )
#endif /* Platform dependent */
		return 1;

	return 0;
}

void
mute_gemtek(void) {
	OUTB(radioport, 0x10);
}

void
volumeup_gemtek(void) {
	OUTB(radioport, 0x20);
}

void
volumedn_gemtek(void) {
	OUTB(radioport, 0x10);
}

static void
send_zero(void) {
	OUTB(radioport, 0x04);
	usleep(5);
	OUTB(radioport, 0x05);
	usleep(5);
}

static void
send_one(void) {
	OUTB(radioport, 0x06);
	usleep(5);
	OUTB(radioport, 0x07);
	usleep(5);
}

static void
send_init(void) {
	OUTB(radioport, 0x03);
	usleep(5);
	OUTB(radioport, 0x07);
	usleep(5);
}

void
Change_Frequency_gemtek(unsigned long freq) {
	int i;
	unsigned long mask = 0x10800ul;

	freq = 78.25*((float)freq/100.0 + 10.52);

	send_init();

	for (i = 0; i < 14; i++)
		if (freq & (1 << i))
			send_one();
		else
			send_zero();

	for ( i = 0; i < 18; i++ )
		if ( mask & (1 << i) )
			send_one();
		else
			send_zero();

	send_init();

	return;
}

int
state_gemtek(void) {
	inb(radioport);
	usleep(5);
	if ( inb(radioport) & 8 )
		return 3;
	return 0;
}
