/*
 * Copyright (c) 2000, 2001 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: io.h,v 1.49 2001/06/05 15:26:57 pva Exp $
 */

#ifndef IO_H__
#define IO_H__

#ifdef linux
#define OUTW(a, b)	outw(b, a)
#define OUTB(a, b)	outb(b, a)
#else
#define OUTW(a, b)	outw(a, b)
#define OUTB(a, b)	outb(a, b)
#endif /* linux */

#define AFC_DELAY	150000

#define SEARCH_PROBE	15
#define SEARCH_LENGTH	19

typedef struct _radiocard_t {
	char *shname; /* Short name of driver */
#define MAX_SHNAME_LEN	31
	unsigned *ports;
	int info;
#define INFO_ROOT	0x80 /* driver needs root privs */
#define INFO_SCAN	0x40 /* detection is done by scanning */
#define INFO_VARS	0x3F /* number of ports in the driver */
	char *name;	/* Name of card */
	unsigned lfreq;	/* Minimal allowed frequency */
	unsigned hfreq;	/* Maximal allowed frequency */
	int (*get_port)(unsigned long);
	int (*free_port)(void);
	void (*set_frequency)(unsigned long);
	void (*mute)(void);
	void (*volume_up)(void);
	void (*volume_down)(void);
	int (*state)(void);
	void (*mono)(void);
	unsigned long (*search_up)(unsigned long);
	unsigned long (*search_down)(unsigned long);
} radiocard_t;

/* Aztech */
int Grab_Port_aztech(unsigned long);
int Release_Port_aztech(void);
void Change_Frequency_aztech(unsigned long);
void mute_aztech(void);
void volumeup_aztech(void);
void volumedn_aztech(void);
int state_aztech(void);
void mono_aztech(void);

/* Zoltrix */
int Grab_Port_zoltrix(unsigned long);
int Release_Port_zoltrix(void);
void Change_Frequency_zoltrix(unsigned long);
void mute_zoltrix(void);
void volumeup_zoltrix(void);
void volumedn_zoltrix(void);
int state_zoltrix(void);
void mono_zoltrix(void);

/* RadioTrack */
int Grab_Port_radiotrack(unsigned long);
int Release_Port_radiotrack(void);
void Change_Frequency_radiotrack(unsigned long);
void mute_radiotrack(void);
void volumeup_radiotrack(void);
void volumedn_radiotrack(void);
int state_radiotrack(void);
void mono_radiotrack(void);

/* GemTek */
int Grab_Port_gemtek(unsigned long);
int Release_Port_gemtek(void);
void Change_Frequency_gemtek(unsigned long);
void mute_gemtek(void);
void volumeup_gemtek(void);
void volumedn_gemtek(void);
int state_gemtek(void);

/* RadioTrack II */
int Grab_Port_radiotrackII(unsigned long);
int Release_Port_radiotrackII(void);
void Change_Frequency_radiotrackII(unsigned long);
void mute_radiotrackII(void);
void unmute_radiotrackII(void);
int state_radiotrackII(void);
void mono_radiotrackII(void);
unsigned long search_up_radiotrackII(unsigned long);
unsigned long search_down_radiotrackII(unsigned long);

/* Maxi Radio */
int Grab_Port_maxiradio(unsigned long);
int Release_Port_maxiradio(void);
void Change_Frequency_maxiradio(unsigned long);
void mute_maxiradio(void);
void unmute_maxiradio(void);
int state_maxiradio(void);

/* Brooktree Bt848 Driver */
int Grab_Port_bktr(unsigned long);
int Release_Port_bktr(void);
void Change_Frequency_bktr(unsigned long);
void mute_bktr(void);
void volumeup_bktr(void);
void volumedn_bktr(void);
int state_bktr(void);
void mono_bktr(void);

/* SoundForte Legacy 128 SF16-FMD2 */
int Grab_Port_sf16fmd2(unsigned long);
int Release_Port_sf16fmd2(void);
void Change_Frequency_sf16fmd2(unsigned long);
void mute_sf16fmd2(void);
void volumeup_sf16fmd2(void);
void volumedn_sf16fmd2(void);
int state_sf16fmd2(void);
void mono_sf16fmd2(void);

/* SoundForte RadioLink SF16-FMR */
int Grab_Port_sf16fmr(unsigned long);
int Release_Port_sf16fmr(void);
void Change_Frequency_sf16fmr(unsigned long);
void mute_sf16fmr(void);
void volumeup_sf16fmr(void);
void volumedn_sf16fmr(void);
int state_sf16fmr(void);

/* SoundForte RadioLink SF16-FMR2 */
int Grab_Port_sf16fmr2(unsigned long);
int Release_Port_sf16fmr2(void);
void Change_Frequency_sf16fmr2(unsigned long);
void mute_sf16fmr2(void);
void unmute_sf16fmr2(void);
int state_sf16fmr2(void);
void mono_sf16fmr2(void);
unsigned long search_up_sf16fmr2(unsigned long);
unsigned long search_down_sf16fmr2(unsigned long);

void range(unsigned, unsigned *, unsigned *, unsigned);
void draw_stick(int);
void mono_stub(void);
unsigned long search_stub(unsigned long);

#endif /* IO_H__ */
