/*
 * Copyright (c) 2000, 2001 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: radiotrack.c,v 1.31 2001/05/15 09:54:54 pva Exp $
 * BETA
 */

#include <unistd.h>

#ifdef linux
#include <sys/io.h>
#elif defined __FreeBSD__
#include <fcntl.h>
#include <machine/cpufunc.h>
#else
#include <machine/pio.h>
#include <machine/sysarch.h>
#endif

#include "io.h"

#ifdef __FreeBSD__
static int iofd;
#endif /* __FreeBSD__ */

static unsigned long radioport = 0;
static int stereo = 1;

static void send_zero(void);
static void send_one(void);

/******************************************************************/

int
Grab_Port_radiotrack(unsigned long port) {
#if defined (__OpenBSD__) || defined (__NetBSD__)
	unsigned long iomap[32];
	int offset;
	unsigned long mask;
	struct i386_set_ioperm_args ioperms;
#endif /* __OpenBSD__ || __NetBSD__ */

	radioport = port;

#ifdef linux /* Platform dependent */
	if ( ioperm(radioport, 2, 1) < 0 )
#elif defined __FreeBSD__
	if ( (iofd = open("/dev/io", O_RDONLY)) < 0 )
#else
	offset = radioport/32;
	mask = 3 << radioport%32;
	memset(iomap, 0xFFFF, sizeof(iomap));
	iomap[offset] ^= mask;
	ioperms.iomap = iomap;
	if ( sysarch(I386_SET_IOPERM, (char *)&ioperms) < 0 )
#endif /* Platform dependent */
		return 1;

	return 0;
}

int
Release_Port_radiotrack(void) {
#if defined (__OpenBSD__) || defined (__NetBSD__)
	unsigned long iomap[32];
	struct i386_set_ioperm_args ioperms;
#endif /* __OpenBSD__ || __NetBSD__ */

#ifdef linux /* Platform dependent */
	if ( ioperm(radioport, 2, 0) < 0 )
#elif defined __FreeBSD__
	if ( close(iofd) < 0 )
#else
	memset(iomap, 0xFFFF, sizeof(iomap));
	ioperms.iomap = iomap;
	if ( sysarch(I386_SET_IOPERM, (char *)&ioperms) < 0 )
#endif /* Platform dependent */
		return 1;

	return 0;
}

void
Change_Frequency_radiotrack(unsigned long frequency) {
	int i;
	int rf = 50; /* Bandwidth 50 kHz */

	frequency += 1070;
	frequency *= 10;
	frequency /= rf;

	/* Frequency */
	for ( i = 0 ; i < 14 ; i++ )
		if ( frequency & (1 << i) )
			send_one();
		else
			send_zero();

	/* LSI test bits */
	send_zero();
	send_zero();

	/* Band */
	send_one();
	if ( stereo )
		send_zero();
	else
		send_one();
	send_zero();

	/* Time Base */
	send_zero();

	/* Reference frequency */
	switch ( rf ) {
		case 25:
			send_zero();
			send_one();
			send_zero();
			break;
		case 100:
			send_zero();
			send_zero();
			send_zero();
			break;
		case 50:
		default:
			send_zero();
			send_zero();
			send_one();
			break;
	}

	/* Do FM */
	send_one();

	usleep(1000);
	OUTB(radioport, 0x10);
	usleep(50000);
	OUTB(radioport, 0xd8);
	return;
}

void
mute_radiotrack(void) {
	/* Mute the card */
	OUTB(radioport, 0x58);
	/* Make sure it's totally down */
	usleep(900000);
	OUTB(radioport, 0xd8);
	/* Turn it off */
	OUTB(radioport, 0x00);
	OUTB(radioport, 0x00);
}

void
volumeup_radiotrack(void) {
	OUTB(radioport, 0x98);
	usleep(100000);
	OUTB(radioport, 0xd8);
}

void
volumedn_radiotrack(void) {
	OUTB(radioport, 0x58);
	usleep(100000);
	OUTB(radioport, 0xd8);
}

int
state_radiotrack(void) {
	int res;

	OUTB(radioport, 0xf8);
	usleep(150000);
	res = (int)inb(radioport);

	if ( res == 0x0fd )
		return 0;
	else
		if ( res != 0xff )
			return 2;

	return 3;
}

void
mono_radiotrack(void) {
	stereo = 0;
}

static void
send_zero(void) {
	OUTB(radioport, 0xd1);
	OUTB(radioport, 0xd1);
	OUTB(radioport, 0xd3);
	OUTB(radioport, 0xd3);
}

static void
send_one(void) {
	OUTB(radioport, 0xd5);
	OUTB(radioport, 0xd5);
	OUTB(radioport, 0xd7);
	OUTB(radioport, 0xd7);
}
