/*
 * Copyright (c) 2001 Vladimir Popov <jumbo@narod.ru>.
 * Copyright (c) 1999 Andy Ramensky, andy@clhs.kiev.ua
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: sf16fmd2.c,v 1.14 2001/05/16 22:55:59 pva Exp $
 * ALPHA
 * lowlevel driver io functions for fmio utility -- sf16fmd2 driver
 */

#include <unistd.h>

#ifdef linux
#include <sys/io.h>
#elif defined __FreeBSD__
#include <fcntl.h>
#include <machine/cpufunc.h>
#else
#include <machine/pio.h>
#include <machine/sysarch.h>
#endif

#include "io.h"

static int stereo = 1;
static unsigned long radioport = 0;
#ifdef __FreeBSD__
static int fdio;
#endif /* __FreeBSD__ */

static void inbits(int);
static void send_zero(int);
static void send_one(int);

/******************************************************************/

int
Grab_Port_sf16fmd2(unsigned long port) {
#if defined (__OpenBSD__) || defined (__NetBSD__) /* __OpenBSD__ || __NetBSD__ */
	unsigned long iomap[32];
	int offset;
	unsigned long mask;
	struct i386_set_ioperm_args ioperms;
#endif /* __OpenBSD__ || __NetBSD__ */

	radioport = port;

#ifdef linux /* Platform dependent */
	if ( ioperm(radioport, 1, 1) < 0 )
#elif defined __FreeBSD__
	if ((fdio = open( "/dev/io", O_RDONLY )) < 0 )
#else
	offset = radioport/32;
	mask = 1 << radioport%32;
	memset(iomap, 0xFFFF, sizeof(iomap));
	iomap[offset] ^= mask;
	ioperms.iomap = iomap;
	if ( sysarch(I386_SET_IOPERM, (char *)&ioperms) < 0 )
#endif /* Platform dependent */
		return 1;

	return 0;
}

int
Release_Port_sf16fmd2(void) {
#if defined (__OpenBSD__) || defined (__NetBSD__) /* __OpenBSD__ || __NetBSD__ */
	unsigned long iomap[32];
	struct i386_set_ioperm_args ioperms;
#endif /* __OpenBSD__ || __NetBSD__ */

#ifdef linux /* Platform dependent */
	if ( ioperm(radioport, 1, 0) < 0 )
#elif defined __FreeBSD__
	if ( close(fdio) < 0 )
#else
	memset(iomap, 0xFFFF, sizeof(iomap));
	ioperms.iomap = iomap;
	if ( sysarch(I386_SET_IOPERM, (char *)&ioperms) < 0 )
#endif /* Platform dependent */
		return 1;

	return 0;
}

void
Change_Frequency_sf16fmd2(unsigned long frequency) {
	int c = 0x0f;
	unsigned long freq = (unsigned long)
		((float)frequency*0.7985714+871.28571);

	/* Search end - station found */
	send_zero(3);

	/* Search down */
	send_zero(3);

	/* Stereo/Forced Mono */
	if ( stereo )
		send_zero(3);
	else
		send_one(3);

	/* FM band */
	send_zero(3);
	send_zero(3);

	/* Band switch */
	send_zero(3);
	send_zero(3);

	/* Locking field strength during search > 30 mkV */
	send_one(3);
	send_zero(3);

	/* Dummy */
	send_zero(3);

	while ( c-- )
		if ( freq & (1<<c) )
			send_one(2);
		else
			send_zero(2);

	usleep(AFC_DELAY);
	return;
}

void
mute_sf16fmd2(void) {
	OUTB(radioport, 0x00);
	inbits(4);
}

void
volumeup_sf16fmd2(void) {
	OUTB(radioport, 0x04);
	inbits(4);
}

void
volumedn_sf16fmd2(void) {
	OUTB(radioport, 0x00);
	inbits(4);
}

int
state_sf16fmd2(void) {
	/* FIXME: stub */
	return 3;
}

static void
inbits(int c) {
	while ( c-- )
		inb(radioport);
}

static void
send_one(int c) {
	OUTB(radioport, 0x01);
	inbits(c);
	OUTB(radioport, 0x03);
	inbits(c);
	OUTB(radioport, 0x01);
	inbits(c);
}

static void
send_zero(int c) {
	OUTB(radioport, 0x00);
	inbits(c);
	OUTB(radioport, 0x02);
	inbits(c);
	OUTB(radioport, 0x00);
	inbits(c);
}

void
mono_sf16fmd2(void) {
	stereo = 0;
}
