/*
 * Copyright (c) 2001 Vladimir Popov <jumbo@narod.ru>.
 * Copyright (c) 2000, 2001 Wilmer van der Gaast <lintux@lintux.cx>
 * Copyright (c) 1998 Kevin Boone <k.boone@mdx.ac.uk>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: sf16fmr.c,v 1.11 2001/05/26 20:28:18 pva Exp $
 * lowlevel io functions for fmio utility -- sf16-fmr driver
 */

#include <unistd.h>

#ifdef linux
#include <sys/io.h>
#elif defined __FreeBSD__
#include <fcntl.h>
#include <machine/cpufunc.h>
#else
#include <machine/pio.h>
#include <machine/sysarch.h>
#endif

#include "io.h"
/*
 * It seems sf16-fmr uses some kind of i2c protocol
 * not quite compatible with the Philips' specification, though.
 */
#if 0
#include "i2c.h"
#endif /* 0 */

	/* Bits representation:
	 * 0 - FREQUENCY SDA
	 * 1 - FREQUENCY SCL
	 * 2 - WREN ?
	 * 3 - unused, always 0
	 * 4 - VOLUME SCL ?
	 * 5 - VOLUME SDA ?
	 * 6 - unused, always 0
	 * 7 - unused, always 0
	 */

const int vtbl[] =	{ 0x0, 0x84, 0x90, 0x104,
			  0x110, 0x204, 0x210, 0x402,
			  0x404, 0x408, 0x410, 0x801,
			  0x802, 0x804, 0x808, 0x810};

static int vol = 0;
static unsigned long radioport = 0;
#ifdef __FreeBSD__
static int iofd;
#endif /* __FreeBSD__ */

static void set_vol(int);
static void send_vol_bit(int);
static void send_freq_bit(int);

/******************************************************************/

int
Grab_Port_sf16fmr(unsigned long port) {
#if defined (__OpenBSD__) || defined (__NetBSD__)
	unsigned long iomap[32];
	int offset;
	unsigned long mask;
	struct i386_set_ioperm_args ioperms;
#endif /* __OpenBSD__ || __NetBSD__ */

	radioport = port;

#ifdef linux /* Platform dependent */
	if ( ioperm(radioport, 4, 1) < 0 )
#elif defined __FreeBSD__
	if ( (iofd = open("/dev/io", O_RDONLY)) < 0 )
#else
	offset = radioport/32;
	mask = 0x0F << radioport%32;
	memset(iomap, 0xFFFF, sizeof(iomap));
	iomap[offset] ^= mask;
	ioperms.iomap = iomap;
	if ( sysarch(I386_SET_IOPERM, (char *)&ioperms) < 0 )
#endif /* Platform dependent */
		return 1;

	/* Reset card */
	//Change_Frequency_sf16fmr(0);
	
	return 0;
}

int
Release_Port_sf16fmr(void) {
#if defined (__OpenBSD__) || defined (__NetBSD__)
	unsigned long iomap[32];
	struct i386_set_ioperm_args ioperms;
#endif /* __OpenBSD__ || __NetBSD__ */

#ifdef linux /* Platform dependent */
	if ( ioperm(radioport, 4, 0) < 0 )
#elif defined __FreeBSD__
	if ( close(iofd) < 0 )
#else
	memset(iomap, 0xFFFF, sizeof(iomap));
	ioperms.iomap = iomap;
	if ( sysarch(I386_SET_IOPERM, (char *)&ioperms) < 0 )
#endif /* Platform dependent */
		return 1;

	return 0;
}

void
Change_Frequency_sf16fmr(unsigned long frequency) {
	int i;
	unsigned long freq = frequency + 1070;
	int data;

	/*
	 * Looks like writing to some i2c compliant PLL
	 * and reading back frequency the PLL is tuned to.
	 */

	/* write START condition ? */
	OUTB(radioport, 0x07);
	OUTB(radioport, 0x03);
	OUTB(radioport, 0x00);
	OUTB(radioport, 0x02);

	data = 0x0b; /* Address of the slave - 0001 011 */
	i = 7;
	while ( i-- )
		if ( data & (1<<i) )
			send_freq_bit(1);
		else
			send_freq_bit(0);

	/* Where's the write bit ? */

	/* Send frequency value */
	for ( i = 0 ; i < 14 ; i++ )
		if ( freq & (1<<i) )
			send_freq_bit(1);
		else
			send_freq_bit(0);

	/* LSI test bits ? */
	send_freq_bit(0);
	send_freq_bit(0);

	/* Band */
	send_freq_bit(0);
	send_freq_bit(1);
	send_freq_bit(0);

	/* Time base */
	send_freq_bit(1);

	/* Reference frequency */
	send_freq_bit(1);
	send_freq_bit(0);
	send_freq_bit(1);

	/* do FM */
	send_freq_bit(0);

	/* STOP condition ? */
	OUTB(radioport, 0x03);

	/* read START condition ? */
	OUTB(radioport, 0x07);
	OUTB(radioport, 0x03);
	OUTB(radioport, 0x01);
	OUTB(radioport, 0x03);

	/* Reading 24 bits - frequency + something ? */
	data = 0x0b; /* the same address - 0001 011 */
	i = 7;
	while ( i-- )
		if ( data & (1<<i) )
			send_freq_bit(1);
		else
			send_freq_bit(0);

	/* Reading ? */
	send_freq_bit(1);

	/* 3 bytes of acknowledgements ?? */
	i = 24;
	while ( i-- )
		send_freq_bit(0);

	OUTB(radioport, 0x03); /* STOP condition ? */
}

void
mute_sf16fmr(void) {
	vol = 0;
	set_vol(vtbl[vol]);
}

void
volumeup_sf16fmr(void) {
	if ( vol < 15 )
		vol++;

	set_vol(vtbl[vol]);
}

void
volumedn_sf16fmr(void) {
	if ( vol > 0 )
		vol--;

	set_vol(vtbl[vol]);
}

int
state_sf16fmr(void) {
	return 3;
}

static void
send_vol_bit(int i) {
	OUTB(radioport, 0x07 + (i<<5));
	OUTB(radioport, 0x17 + (i<<5));
	inb(radioport);
	inb(radioport);
}

static void
send_freq_bit(int i) {
	OUTB(radioport, 0x04 + i);
	OUTB(radioport, 0x06 + i);
	inb(radioport);
	inb(radioport);
}

static void
set_vol(int v) {
	int i = 12;

	/*
	 * Apparently not i2c, some serially controlled amplifier.
	 * But which one ?
	 */

	OUTB(radioport, 0x07);

	while ( i-- )
		if ( v & (1<<i) )
			send_vol_bit(1);
		else
			send_vol_bit(0);

	send_vol_bit(0);
	send_vol_bit(1);
	send_vol_bit(1);
	send_vol_bit(0);
	send_vol_bit(0);
	send_vol_bit(0);

	OUTB(radioport, 0x0f);
}
