/*
 * Copyright (c) 2001 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: specs.h,v 1.6 2001/06/24 15:17:02 pva Exp $
 *
 * specs.h -- database of supported cards
 */

#ifndef SPECS_H__
#define SPECS_H__

#include "io.h"

unsigned aztech_ports[] = {0x350, 0x358};
unsigned radiotrack_ports[] = {0x20c, 0x30c};
unsigned bktr_ports[] = {ERADIO_NONE};
unsigned gemtek_ports[] = {0x20c, 0x30c, 0x24c, 0x34c, 0x248};
unsigned maxiradio_ports[] = {0x6400};
unsigned sf16fm_ports[] = {0x284, 0x384};

const radiocard_t card[] = {
	{ "az", aztech_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"Aztech/PackardBell FM Radio Card", 8750, 10800,
	Grab_Port_aztech, Release_Port_aztech,
	Change_Frequency_aztech,
	mute_aztech, volumeup_aztech, volumedn_aztech,
	state_aztech,
	mono_aztech,
	search_up_generic, search_down_generic},

	{ "zx", radiotrack_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"Zoltrix Radio Plus FM Radio Card", 8800, 10800,
	Grab_Port_zoltrix, Release_Port_zoltrix,
	Change_Frequency_zoltrix,
	mute_zoltrix, volumeup_zoltrix, volumedn_zoltrix,
	state_zoltrix,
	mono_zoltrix,
	search_up_generic, search_down_generic},

	{ "rt", radiotrack_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"AIMS Lab RadioTrack/RadioReveal FM Radio Card", 8750, 10800,
	Grab_Port_radiotrack, Release_Port_radiotrack,
	Change_Frequency_radiotrack,
	mute_radiotrack, volumeup_radiotrack, volumedn_radiotrack,
	state_radiotrack,
	mono_radiotrack,
	search_up_generic, search_down_generic},

	{ "bktr", bktr_ports, 0x01,
#ifdef linux
	"Video4Linux Driver",
#else
	"Brooktree Bt848 Driver",
#endif /* linux */
	8750, 10800,
	Grab_Port_bktr, Release_Port_bktr,
	Change_Frequency_bktr,
	mute_bktr, volumeup_bktr, volumedn_bktr,
	state_bktr,
	mono_bktr,
	search_up_generic, search_down_generic},

	{ "gt", gemtek_ports, INFO_ROOT | INFO_SCAN | 0x05,
	"GemTek FM Radio Card", 8700, 10800,
	Grab_Port_gemtek, Release_Port_gemtek,
	Change_Frequency_gemtek,
	mute_gemtek, volumeup_gemtek, volumedn_gemtek,
	state_gemtek,
	mono_stub,
	search_up_generic, search_down_generic},

	{ "rtII", radiotrack_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"AIMS Lab RadioTrack II FM Radio Card", 8750, 10800,
	Grab_Port_radiotrackII, Release_Port_radiotrackII,
	Change_Frequency_radiotrackII,
	mute_radiotrackII, unmute_radiotrackII, mute_radiotrackII,
	state_radiotrackII,
	mono_radiotrackII,
	search_up_radiotrackII, search_down_radiotrackII},

	{ "sf2d", sf16fm_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"SoundForte Legacy 128 SF16-FMD2 Driver", 8750, 10800,
	Grab_Port_sf16fmd2, Release_Port_sf16fmd2,
	Change_Frequency_sf16fmd2,
	mute_sf16fmd2, volumeup_sf16fmd2, volumedn_sf16fmd2,
	state_sf16fmd2,
	mono_sf16fmd2,
	search_stub, search_stub},

	{ "sfr", sf16fm_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"SoundForte RadioLink SF16-FMR FM Radio Card", 8750, 10800,
	Grab_Port_sf16fmr, Release_Port_sf16fmr,
	Change_Frequency_sf16fmr,
	mute_sf16fmr, volumeup_sf16fmr, volumedn_sf16fmr,
	state_sf16fmr,
	mono_stub,
	search_stub, search_stub},

	{ "sf2r", sf16fm_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"SoundForte RadioLink SF16-FMR2 FM Radio Card", 8750, 10800,
	Grab_Port_sf16fmr2, Release_Port_sf16fmr2,
	Change_Frequency_sf16fmr2,
	mute_sf16fmr2, unmute_sf16fmr2, mute_sf16fmr2,
	state_sf16fmr2,
	mono_sf16fmr2,
	search_up_sf16fmr2, search_down_sf16fmr2},

	{ "sfi", sf16fm_ports, INFO_ROOT | INFO_SCAN | 0x02,
	"SoundForte RadioX SF16-FMI Driver", 8750, 10800,
	Grab_Port_radiotrack, Release_Port_radiotrack,
	Change_Frequency_radiotrack,
	mute_radiotrack, volumeup_radiotrack, volumedn_radiotrack,
	state_radiotrack,
	mono_radiotrack,
	search_up_generic, search_down_generic},

	{ "mr", maxiradio_ports, INFO_ROOT | INFO_SCAN | 0x01,
	"Guillemot Maxi Radio FM 2000 PCI Radio Card", 8750, 10800,
	Grab_Port_maxiradio, Release_Port_maxiradio,
	Change_Frequency_maxiradio,
	mute_maxiradio, unmute_maxiradio, mute_maxiradio,
	state_maxiradio,
	mono_stub,
	search_up_generic, search_down_generic}
};

#endif /* SPECS_H__ */
