/*
 * Copyright (c) 2001 Vladimir Popov <jumbo@narod.ru>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * $Id: tea5757.c,v 1.7 2001/06/05 14:31:17 pva Exp $
 * implementation of routines for TEA5757 chip
 */

#include <unistd.h>

#include "io.h"
#include "tea5757.h"

void
tea5757_write_shift_register(struct tea5757_t *card) {
	unsigned long reg = 0ul;

	if ( card->frequency ) {
		reg = card->frequency;
		reg += 1070;
		reg /= 1.25;
	} else {
		reg = TEA5757_SEARCH_START | card->search;
	}

	reg |= card->stereo;
	reg |= card->sensitivity;

	card->write(reg);

	return;
}

unsigned long
tea5757_read_shift_register(struct tea5757_t *card) {
	unsigned long reg;

	/* FIXME: do something about it */
	usleep(AFC_DELAY);
	reg = card->read();

	return reg;
}

unsigned long
tea5757_search(struct tea5757_t *card) {
	unsigned long tmp = card->search;

	card->search = TEA5757_SEARCH_END;
	tea5757_write_shift_register(card);

	card->frequency = 0;
	card->search = tmp;
	tea5757_write_shift_register(card);

	tmp = tea5757_read_shift_register(card);

	return tea5757_decode_frequency(tmp);
}

unsigned long
tea5757_decode_frequency(unsigned long data) {
	unsigned long freq = data & TEA5757_FREQ;

	freq *= 1.25;
	freq -= 1070;

	return freq;
}
